@@ -1,51 +1,50 @@
-{-# LANGUAGE Rank2Types, CPP #-}
+{-# LANGUAGE Rank2Types, CPP, FlexibleContexts #-}
 module Categorize where
 
 import Data
 
 import qualified Text.Regex.PCRE.Light.Text as RE
-import qualified Data.Map as M
 import qualified Data.MyText as T
 import Data.MyText (Text)
+import Control.Applicative (empty, (<*))
 import Control.Monad
-import Control.Monad.Instances
-import Control.Monad.Trans.Reader
+import Control.Monad.Instances()
+import Control.Monad.Trans.Reader hiding (local)
+import Control.Monad.Reader.Class (local)
 import Control.Monad.Trans.Class
 import Data.Functor.Identity
 
-
-import Text.Parsec
-import Text.Parsec.Char
-import Text.Parsec.Token
-import Text.Parsec.Combinator
-import Text.Parsec.Language
-import Text.Parsec.ExprFail
-import System.IO
-import System.Exit
-import Control.Applicative ((<*>),(<$>))
+import Control.Applicative ((<$>))
 import Control.DeepSeq
-import Data.List
-import Data.Maybe
 import Data.Char
-import Data.Time.Clock
-import Data.Time.LocalTime
+import Data.List
+import Data.Map.Strict (Map)
+import qualified Data.Map.Strict as Map
+import Data.Maybe
 import Data.Time.Calendar (toGregorian, fromGregorian)
 import Data.Time.Calendar.WeekDate (toWeekDate)
+import Data.Time.Clock
 import Data.Time.Format (formatTime)
+import Data.Time.LocalTime
+import System.Exit
+import System.IO
+import Text.Show.Functions
+import Text.Parsec
+import Text.Parsec.ExprFail
+import Text.Parsec.Token
 #if MIN_VERSION_time(1,5,0)
 import Data.Time.Format(defaultTimeLocale, iso8601DateFormat)
 #else
 import System.Locale (defaultTimeLocale, iso8601DateFormat)
 #endif
 import Debug.Trace
-import Control.Arrow (second)
 import Text.Printf
 
 type Categorizer = TimeLog CaptureData -> TimeLog (Ctx, ActivityData)
 type Rule = Ctx -> ActivityData
+type Environment = Map String Cond
 
-type Parser = ParsecT String () (ReaderT TimeZone Identity)
-
+type Parser = ParsecT String () (ReaderT (TimeZone, Environment) Identity)
 
 data Ctx = Ctx
         { cNow :: TimeLogEntry CaptureData
@@ -54,11 +53,11 @@ data Ctx = Ctx
         , cSubsts :: [Text]
         , cCurrentTime :: UTCTime
         , cTimeZone :: TimeZone
-        }
-  deriving (Show)
+        , conditionBindings :: Map String Cond
+        } deriving Show
 
 instance NFData Ctx where
-    rnf (Ctx a b c d e f) = a `deepseq` b `deepseq` c `deepseq` e `deepseq` e `deepseq` f `deepseq` ()
+    rnf (Ctx a b c d e f g) = a `deepseq` b `deepseq` c `deepseq` d `deepseq` e `deepseq` f `deepseq` g `deepseq` ()
 
 type Cond = CtxFun [Text]
 
@@ -76,31 +75,44 @@ data CondPrim
 
 newtype Cmp = Cmp (forall a. Ord a => a -> a -> Bool)
 
+data DateVar = DvDate | DvNow
+
+data TimeVar = TvTime | TvSampleAge
+
+data NumVar = NvIdle
+
+runParserStack :: Stream s (ReaderT r Identity) t
+               => r
+               -> ParsecT s () (ReaderT r Identity) a
+               -> SourceName
+               -> s
+               -> Either ParseError a
+runParserStack env p filename =
+  runIdentity . flip runReaderT env . runParserT p () filename
+
 readCategorizer :: FilePath -> IO Categorizer
-readCategorizer filename = do
-        h <- openFile filename ReadMode
+readCategorizer filename = withFile filename ReadMode $ \h -> do
         hSetEncoding h utf8
         content <- hGetContents h
         time <- getCurrentTime
         tz <- getCurrentTimeZone
-        case flip runReader tz $
-            runParserT (between (return ()) eof parseRules) () filename content of
+        case runParserStack (tz, Map.empty) (between (return ()) eof parseRules) filename content of
           Left err -> do
                 putStrLn "Parser error:"
                 print err
                 exitFailure
-          Right cat -> return $
+          Right cat -> return
                 (map (fmap (mkSecond (postpare . cat))) . prepare time tz)
 
-applyCond :: String -> TimeZone -> TimeLogEntry (Ctx, ActivityData) -> Bool
-applyCond s tz = 
-        case flip runReader tz $ runParserT (do {c <- parseCond; eof ; return c}) () "commad line parameter" s of
+applyCond :: String -> TimeZone -> Environment -> TimeLogEntry (Ctx, ActivityData) -> Bool
+applyCond s tz env =
+        case runParserStack (tz, env) (parseCond <* eof) "command line parameter" s of
           Left err -> error (show err)
           Right c  -> isJust . c . fst . tlData
 
 prepare :: UTCTime -> TimeZone -> TimeLog CaptureData -> TimeLog Ctx
 prepare time tz = map go
-  where go now  = now {tlData = Ctx now (findActive (cWindows (tlData now))) Nothing [] time tz }
+  where go now  = now {tlData = Ctx now (findActive (cWindows (tlData now))) Nothing [] time tz Map.empty }
 
 -- | Here, we filter out tags appearing twice, and make sure that only one of
 --   each category survives
@@ -109,23 +121,32 @@ postpare = nubBy go
   where go (Activity (Just c1) _) (Activity (Just c2) _) = c1 == c2
         go a1                     a2                     = a1 == a2
 
-lang :: GenTokenParser String () (ReaderT TimeZone Identity)
-lang = makeTokenParser $ LanguageDef
+lang :: GenTokenParser String () (ReaderT (TimeZone, Environment) Identity)
+lang = makeTokenParser LanguageDef
                 { commentStart   = "{-"
                 , commentEnd     = "-}"
                 , commentLine    = "--"
                 , nestedComments = True
                 , identStart     = letter
-                , identLetter	 = alphaNum <|> oneOf "_'"
-                , opStart	 = oneOf ":!#$%&*+./<=>?@\\^|-~"
-                , opLetter	 = oneOf ":!#$%&*+./<=>?@\\^|-~"
+                , identLetter    = alphaNum <|> oneOf "_'"
+                , opStart        = oneOf ":!#$%&*+./<=>?@\\^|-~"
+                , opLetter       = oneOf ":!#$%&*+./<=>?@\\^|-~"
                 , reservedOpNames= []
-                , reservedNames  = []
+                , reservedNames  = [ "title"
+                                   , "program"
+                                   , "active"
+                                   , "idle"
+                                   , "time"
+                                   , "sampleage"
+                                   , "date"
+                                   , "now"
+                                   , "desktop"
+                                   ]
                 , caseSensitive  = True
                 }
 
 parseRules :: Parser Rule
-parseRules = do 
+parseRules = do
         whiteSpace lang
         a <- option id (reserved lang "aliases" >> parens lang parseAliasSpecs)
         rb <- parseRulesBody
@@ -147,17 +168,29 @@ parseAliasSpec = do s1 <- T.pack <$> stringLiteral lang
                     return (s1,s2)
 
 parseRulesBody :: Parser Rule
-parseRulesBody = do 
+parseRulesBody = do
         x <- parseRule
-        choice [ do comma lang
+        choice [ do _ <- comma lang
                     xs <- parseRule `sepEndBy1` comma lang
                     return (matchAny (x:xs))
-               , do semi lang
+               , do _ <- semi lang
                     xs <- parseRule `sepEndBy1` semi lang
                     return (matchFirst (x:xs))
                ,    return x
                ]
 
+withBinding :: String -> Cond -> Parser a -> Parser a
+withBinding k v = local (\(tz,env) -> (tz, Map.insert k v env))
+
+parseConditionBinding :: Parser Rule
+parseConditionBinding = do
+  _ <- reserved lang "condition"
+  varname <- identifier lang
+  _ <- reservedOp lang "="
+  cond <- parseCond
+  _ <- reserved lang "in"
+  withBinding varname cond parseRule
+
 parseRule :: Parser Rule
 parseRule = choice
         [    braces lang parseRules
@@ -174,13 +207,14 @@ parseRule = choice
              return (ifThenElse cond rule1 rule2)
         , do reserved lang "tag"
              parseSetTag
+        , parseConditionBinding
         ]
 
 parseCond :: Parser Cond
 parseCond = do cp <- parseCondExpr
                case cp of
                 CondCond c -> return c
-                cp         -> fail $ printf "Expression of type %s" (cpType cp)
+                _          -> fail $ printf "Expression of type %s" (cpType cp)
 
 parseCondExpr :: Parser CondPrim
 parseCondExpr = buildExpressionParser [
@@ -190,7 +224,7 @@ parseCondExpr = buildExpressionParser [
                 , Prefix (reserved lang "month" >> return evalMonth)
                 , Prefix (reserved lang "year" >> return evalYear)
                 , Prefix (reserved lang "format" >> return formatDate) ],
-                [ Infix (reservedOp lang "=~" >> return checkRegex) AssocNone 
+                [ Infix (reservedOp lang "=~" >> return checkRegex) AssocNone
                 , Infix (checkCmp <$> parseCmp) AssocNone
                 ],
                 [ Prefix (reserved lang "current window" >> return checkCurrentwindow)
@@ -242,8 +276,7 @@ checkOr cp1 cp2 = Left $
                (cpType cp1) (cpType cp2)
 
 checkNot :: CondPrim -> Erring CondPrim
-checkNot (CondCond getCnd) = Right $ CondCond $ do
-        liftM (maybe (Just []) (const Nothing)) getCnd
+checkNot (CondCond getCnd) = Right . CondCond $ fmap (maybe (Just []) (const Nothing)) getCnd
 checkNot cp = Left $
         printf "Cannot apply ! to an expression of type %s"
                (cpType cp)
@@ -279,7 +312,7 @@ checkCmp _ cp1 cp2 = Left $
                (cpType cp1) (cpType cp2)
 
 checkCurrentwindow :: CondPrim -> Erring CondPrim
-checkCurrentwindow (CondCond cond) = Right $ CondCond $ \ctx -> 
+checkCurrentwindow (CondCond cond) = Right $ CondCond $ \ctx ->
         cond (ctx { cWindowInScope = cCurrentWindow ctx })
 checkCurrentwindow cp = Left $
         printf "Cannot apply current window to an expression of type %s"
@@ -293,15 +326,20 @@ checkAnyWindow cp = Left $
         printf "Cannot apply current window to an expression of type %s"
                (cpType cp)
 
+fst3 :: (a,b,c) -> a
 fst3 (a,_,_) = a
+
+snd3 :: (a,b,c) -> b
 snd3 (_,b,_) = b
+
+trd3 :: (a,b,c) -> c
 trd3 (_,_,c) = c
 
 -- Day of week is an integer in [1..7].
 evalDayOfWeek :: CondPrim -> Erring CondPrim
 evalDayOfWeek (CondDate df) = Right $ CondInteger $ \ctx ->
   let tz = cTimeZone ctx in
-  (toInteger . trd3 . toWeekDate . localDay . utcToLocalTime tz) `liftM` df ctx
+  (toInteger . trd3 . toWeekDate . localDay . utcToLocalTime tz) `fmap` df ctx
 evalDayOfWeek cp = Left $ printf
   "Cannot apply day of week to an expression of type %s, only to $date."
   (cpType cp)
@@ -310,7 +348,7 @@ evalDayOfWeek cp = Left $ printf
 evalDayOfMonth :: CondPrim -> Erring CondPrim
 evalDayOfMonth (CondDate df) = Right $ CondInteger $ \ctx ->
   let tz = cTimeZone ctx in
-  (toInteger . trd3 . toGregorian . localDay . utcToLocalTime tz) `liftM` df ctx
+  (toInteger . trd3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalDayOfMonth cp = Left $ printf
   "Cannot apply day of month to an expression of type %s, only to $date."
   (cpType cp)
@@ -319,7 +357,7 @@ evalDayOfMonth cp = Left $ printf
 evalMonth :: CondPrim -> Erring CondPrim
 evalMonth (CondDate df) = Right $ CondInteger $ \ctx ->
   let tz = cTimeZone ctx in
-  (toInteger . snd3 . toGregorian . localDay . utcToLocalTime tz) `liftM` df ctx
+  (toInteger . snd3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalMonth cp = Left $ printf
   "Cannot apply month to an expression of type %s, only to $date."
   (cpType cp)
@@ -327,7 +365,7 @@ evalMonth cp = Left $ printf
 evalYear :: CondPrim -> Erring CondPrim
 evalYear (CondDate df) = Right $ CondInteger $ \ctx ->
   let tz = cTimeZone ctx in
-  (fst3 . toGregorian . localDay . utcToLocalTime tz) `liftM` df ctx
+  (fst3 . toGregorian . localDay . utcToLocalTime tz) `fmap` df ctx
 evalYear cp = Left $ printf
   "Cannot apply year to an expression of type %s, only to $date."
   (cpType cp)
@@ -336,7 +374,7 @@ evalYear cp = Left $ printf
 formatDate :: CondPrim -> Erring CondPrim
 formatDate (CondDate df) = Right $ CondString $ \ctx ->
   let tz = cTimeZone ctx
-      local = utcToLocalTime tz `liftM` df ctx
+      local = utcToLocalTime tz `fmap` df ctx
    in T.pack . formatTime defaultTimeLocale (iso8601DateFormat Nothing) <$> local
 formatDate cp = Left $ printf
   "Cannot format an expression of type %s, only $date." (cpType cp)
@@ -347,6 +385,7 @@ parseCmp = choice $ map (\(s,o) -> reservedOp lang s >> return o)
                          (">", Cmp (>)),
                          ("==",Cmp (==)),
                          ("=", Cmp (==)),
+                         ("!=",Cmp (/=)),
                          ("<", Cmp (<)),
                          ("<=",Cmp (<=))]
 
@@ -361,28 +400,24 @@ parseCondPrim = choice
                 return $ CondRegexList (const (Just list))
             ) <?> "list of regular expressions"
             ])
-        , char '$' >> choice 
+        , char '$' >> choice
              [ do backref <- read <$> many1 digit
                   return $ CondString (getBackref backref)
-             , do varname <- identifier lang 
-                  choice 
-                      [ do guard $ varname == "title"
-                           return $ CondString (getVar "title")
-                      , do guard $ varname == "program"
-                           return $ CondString (getVar "program")
-                      , do guard $ varname == "active"
-                           return $ CondCond checkActive
-                      , do guard $ varname == "idle"
-                           return $ CondInteger (getNumVar "idle")
-                      , do guard $ varname == "time"
-                           return $ CondTime (getTimeVar "time")
-                      , do guard $ varname == "sampleage"
-                           return $ CondTime (getTimeVar "sampleage")
-                      , do guard $ varname == "date"
-                           return $ CondDate (getDateVar "date")
-                      , do guard $ varname == "desktop"
-                           return $ CondString (getVar "desktop")
-                     ]
+             , choice [ reserved lang "title" >> return (CondString (getVar "title"))
+                      , reserved lang "program" >> return (CondString (getVar "program"))
+                      , reserved lang "active" >> return (CondCond checkActive)
+                      , reserved lang "idle" >> return (CondInteger (getNumVar NvIdle))
+                      , reserved lang "time" >> return (CondTime (getTimeVar TvTime))
+                      , reserved lang "sampleage" >> return (CondTime (getTimeVar TvSampleAge))
+                      , reserved lang "date" >> return (CondDate (getDateVar DvDate))
+                      , reserved lang "now" >> return (CondDate (getDateVar DvNow))
+                      , reserved lang "desktop" >> return (CondString (getVar "desktop"))
+                      , do varname <- identifier lang
+                           inEnvironment <- (lift (asks (Map.lookup varname . snd)))
+                           case inEnvironment of
+                             Nothing -> fail ("Reference to unbound variable: '" ++ varname ++ "'")
+                             Just cond -> return (CondCond cond)
+                      ]
               ] <?> "variable"
         , do regex <- parseRegex <?> "regular expression"
              return $ CondRegex (const (Just regex))
@@ -395,63 +430,32 @@ parseCondPrim = choice
         , do num <- natural lang <?> "number"
              return $ CondInteger (const (Just num))
         ]
-{-
-                     choice
-                        [ do reservedOp lang "=~"
-                             regex <- parseRegex
-                             return $ checkRegex varname (RE.compile regex [])
-                        , do reservedOp lang "==" <|> reservedOp lang "="
-                             str <- stringLiteral lang
-                             return $ checkEq varname str
-                        , do reservedOp lang "/=" <|> reservedOp lang "!="
-                             str <- stringLiteral lang
-                             return $ checkNot (checkEq varname str)
-                        ]
-                , do guard $ varname == "idle"
-                     op <- parseCmp
-                     num <- natural lang
-                     return $ checkNumCmp op varname num
-                , do guard $ varname `elem` ["time","sampleage"]
-                     op <- parseCmp 
-                     time <- parseTime
-                     return $ checkTimeCmp op varname time
-                , do guard $ varname == "active"
-                     return checkActive
-                ]
-        , do reserved lang "current window"
-             cond <- parseCond
-             return $ checkCurrentwindow cond
-        , do reserved lang "any window"
-             cond <- parseCond
-             return $ checkAnyWindow cond
-        ]
--}
 
 parseRegex :: Parser RE.Regex
 parseRegex = fmap (flip RE.compile [] . T.pack) $ lexeme lang $ choice
         [ between (char '/') (char '/') (many1 (noneOf "/"))
-        , do char 'm'
+        , do _ <- char 'm'
              c <- anyChar
              str <- many1 (noneOf [c])
-             char c
+             _ <- char c
              return str
         ]
-             
+
 -- | Parses a day-of-time specification (hh:mm)
 parseTime :: Parser NominalDiffTime
 parseTime = fmap fromIntegral $ lexeme lang $ do
                hour <- read <$> many1 digit
-               char ':'
+               _ <- char ':'
                minute <- read <$> count 2 digit
                return $ (hour * 60 + minute) * 60
 
 parseDate :: Parser UTCTime
 parseDate = lexeme lang $ do
-    tz <- lift ask
+    tz <- lift (asks fst)
     year <- read <$> count 4 digit
-    char '-'
+    _ <- char '-'
     month <- read <$> count 2 digit
-    char '-'
+    _ <- char '-'
     day <- read <$> count 2 digit
     time <- option 0 parseTime
     let date = LocalTime (fromGregorian year month day) (TimeOfDay 0 0 0)
@@ -460,7 +464,7 @@ parseDate = lexeme lang $ do
 
 parseSetTag :: Parser Rule
 parseSetTag = lexeme lang $ do
-                 firstPart <- parseTagPart 
+                 firstPart <- parseTagPart
                  choice [ do char ':'
                              secondPart <- parseTagPart
                              return $ do cat <- firstPart
@@ -476,7 +480,7 @@ parseSetTag = lexeme lang $ do
                         ]
 
 replaceForbidden :: Maybe Text -> Maybe Text
-replaceForbidden = liftM $ T.map go
+replaceForbidden = fmap $ T.map go
   where
     go c | isAlphaNum c  = c
          | c `elem` "-_" = c
@@ -498,12 +502,12 @@ parseTagPart = do parts <- many1 (choice
 
 ifThenElse :: Cond -> Rule -> Rule -> Rule
 ifThenElse cond r1 r2 = do res <- cond
-                           case res of 
+                           case res of
                             Just substs -> r1 . setSubsts substs
                             Nothing -> r2
   where setSubsts :: [Text] -> Ctx -> Ctx
         setSubsts substs ctx = ctx { cSubsts = substs }
-        
+
 
 matchAny :: [Rule] -> Rule
 matchAny rules = concat <$> sequence rules
@@ -511,7 +515,7 @@ matchFirst :: [Rule] -> Rule
 matchFirst rules = takeFirst <$> sequence rules
   where takeFirst [] = []
         takeFirst ([]:xs) = takeFirst xs
-        takeFirst (x:xs) = x
+        takeFirst (x:_) = x
 
 
 getBackref :: Integer -> CtxFun Text
@@ -528,27 +532,27 @@ getVar "title"   ctx = do
 getVar "program" ctx = do
                 (_,_,p) <- cWindowInScope ctx
                 return p
-getVar "desktop" ctx = do
-                return $ cDesktop (tlData (cNow ctx))
-getVar v ctx = error $ "Unknown variable " ++ v
+getVar "desktop" ctx = return $ cDesktop (tlData (cNow ctx))
+getVar v _ = error $ "Unknown variable " ++ v
 
-getNumVar :: String -> CtxFun Integer
-getNumVar "idle" ctx = Just $ cLastActivity (tlData (cNow ctx)) `div` 1000
+getNumVar :: NumVar -> CtxFun Integer
+getNumVar NvIdle ctx = Just $ cLastActivity (tlData (cNow ctx)) `div` 1000
 
-getTimeVar :: String -> CtxFun NominalDiffTime
-getTimeVar "time" ctx = Just $
+getTimeVar :: TimeVar -> CtxFun NominalDiffTime
+getTimeVar TvTime ctx = Just $
    let utc = tlTime . cNow $ ctx
        tz = cTimeZone ctx
        local = utcToLocalTime tz utc
        midnightUTC = localTimeToUTC tz $ local { localTimeOfDay = midnight }
     in utc `diffUTCTime` midnightUTC
-getTimeVar "sampleage" ctx = Just $ cCurrentTime ctx `diffUTCTime` tlTime (cNow ctx)
+getTimeVar TvSampleAge ctx = Just $ cCurrentTime ctx `diffUTCTime` tlTime (cNow ctx)
 
-getDateVar :: String -> CtxFun UTCTime
-getDateVar "date" ctx = Just $ tlTime (cNow ctx)
+getDateVar :: DateVar -> CtxFun UTCTime
+getDateVar DvDate = Just . tlTime . cNow
+getDateVar DvNow = Just . cCurrentTime
 
 findActive :: [(Bool, t, t1)] -> Maybe (Bool, t, t1)
-findActive = find (\(a,_,_) -> a)                                 
+findActive = find (\(a,_,_) -> a)
 
 checkActive :: Cond
 checkActive ctx = do (a,_,_) <- cWindowInScope ctx
@@ -560,7 +564,7 @@ matchNone = const []
 
 justIf :: a -> Bool -> Maybe a
 justIf x True = Just x
-justIf x False = Nothing
+justIf _ False = Nothing
 
 mkSecond :: (a -> b) -> a -> (a, b)
 mkSecond f a = (a, f a)
