@@ -22,7 +22,7 @@ import Data.Maybe
 import Data.List
 import Data.Ord
 import Text.Printf
-import qualified Data.Map as M
+import qualified Data.Map.Strict as M
 import qualified Data.Set as S
 import Data.MyText (Text,pack,unpack)
 import Data.Function (on)
@@ -86,7 +86,7 @@ defaultReportOptions = ReportOptions
 
 -- Data format semantically representing the result of a report, including the
 -- title
-type Interval = (String,String,String,String) 
+type Interval = (String,String,String,String)
 data ReportResults =
         ListOfFields String [(String, String)]
         | ListOfTimePercValues String [(String, String, Double)]
@@ -98,8 +98,8 @@ data ReportResults =
 
 
 filterPredicate :: [Filter] -> TimeLogEntry (Ctx, ActivityData) -> Bool
-filterPredicate filters tl = 
-       all (\flag -> case flag of 
+filterPredicate filters tl =
+       all (\flag -> case flag of
                 Exclude act  -> excludeTag act tl
                 Only act     -> onlyTag act tl
                 GeneralCond s-> applyCond s (cTimeZone (fst (tlData tl))) M.empty tl) filters
@@ -110,8 +110,8 @@ filterActivity fs = filter (applyActivityFilter fs)
 applyActivityFilter :: [ActivityFilter] -> Activity -> Bool
 applyActivityFilter fs act = all go fs
     where go (ExcludeActivity matcher) = not (matchActivityMatcher matcher act)
-          go (OnlyActivity matcher)    =      matchActivityMatcher matcher act 
-                                
+          go (OnlyActivity matcher)    =      matchActivityMatcher matcher act
+
 excludeTag matcher = not . any (matchActivityMatcher matcher) . snd . tlData
 onlyTag matcher = any (matchActivityMatcher matcher) . snd . tlData
 
@@ -122,7 +122,7 @@ matchActivityMatcher :: ActivityMatcher -> Activity -> Bool
 matchActivityMatcher (MatchActivity act1) act2 = act1 == act2
 matchActivityMatcher (MatchCategory cat) act2 = Just cat == activityCategory act2
 
-parseActivityMatcher :: String -> ActivityMatcher 
+parseActivityMatcher :: String -> ActivityMatcher
 parseActivityMatcher str | last str == ':' = MatchCategory (pack (init str))
                          | otherwise       = MatchActivity (read str)
 
@@ -145,15 +145,15 @@ data Calculations = Calculations
 
 prepareCalculations :: LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) Calculations
 prepareCalculations =
-    pure (\fd ld ttr tts s -> 
+    pure (\fd ld ttr tts s ->
         let c = Calculations
                   { firstDate = fd
                   , lastDate = ld
                   , timeDiff = diffUTCTime (lastDate c) (firstDate c)
                   , totalTimeRec = ttr
                   , totalTimeSel = tts
-                  , fractionRec = realToFrac (totalTimeRec c) / (realToFrac (timeDiff c))
-                  , fractionSel = realToFrac (totalTimeSel c) / (realToFrac (timeDiff c))
+                  , fractionRec = realToFrac (totalTimeRec c) / realToFrac (timeDiff c)
+                  , fractionSel = realToFrac (totalTimeSel c) / realToFrac (timeDiff c)
                   , fractionSelRec = realToFrac (totalTimeSel c) / realToFrac (totalTimeRec c)
                   , sums = s
                   } in c) <*>
@@ -161,8 +161,7 @@ prepareCalculations =
     onAll calcLastDate <*>
     onAll calcTotalTime <*>
     onSelected calcTotalTime <*>
-    onSelected calcSums 
-  where
+    onSelected calcSums
 
 calcFirstDate :: LeftFold (TimeLogEntry a) UTCTime
 calcFirstDate = fromJust <$> lfFirst `mapElems` tlTime
@@ -176,7 +175,7 @@ calcTotalTime = (/1000) <$> LeftFold 0 (+) fromInteger `mapElems` tlRate
 calcSums :: LeftFold (TimeLogEntry (a, [Activity])) (M.Map Activity NominalDiffTime)
 calcSums = LeftFold M.empty
             (\m tl ->
-                let go' m act = M.insertWith' (+) act (fromInteger (tlRate tl)/1000) m
+                let go' m act = M.insertWith (+) act (fromInteger (tlRate tl)/1000) m
                 in foldl' go' m (snd (tlData tl))) id
 
 processRepeater :: TimeZone -> Repeater -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults -> LeftFold (Bool :!: TimeLogEntry (Ctx, ActivityData)) ReportResults
@@ -218,8 +217,8 @@ processReport :: TimeZone -> ReportOptions -> Report -> LeftFold (Bool :!: TimeL
 processReport tz opts GeneralInfos =
    pure (\n firstDate lastDate ttr tts ->
     let timeDiff = diffUTCTime lastDate firstDate
-        fractionRec = realToFrac ttr / (realToFrac timeDiff) :: Double
-        fractionSel = realToFrac tts / (realToFrac timeDiff) :: Double
+        fractionRec = realToFrac ttr / realToFrac timeDiff :: Double
+        fractionSel = realToFrac tts / realToFrac timeDiff :: Double
         fractionSelRec = realToFrac tts / realToFrac ttr :: Double
     in ListOfFields "General Information"
         [ ("FirstRecord", show firstDate)
@@ -239,43 +238,39 @@ processReport tz opts GeneralInfos =
 
 processReport tz opts  TotalTime =
     onSelected $
-        pure (\totalTimeSel sums -> 
+        pure (\totalTimeSel sums ->
             ListOfTimePercValues "Total time per tag" .
             mapMaybe (\(tag,time) ->
                   let perc = realToFrac time/realToFrac totalTimeSel
                       pick = applyActivityFilter (roActivityFilter opts) tag
                   in if pick && perc*100 >= roMinPercentage opts
-                  then Just $ ( show tag
-                              , showTimeDiff opts time
-                              , perc)
+                  then Just ( show tag , showTimeDiff opts time , perc)
                   else Nothing
                   ) .
-            reverse .
-            sortBy (comparing snd) $
-            M.toList $
-            sums) <*>
+            sortOn (Down . snd) $
+            M.toList sums) <*>
     calcTotalTime <*>
-    calcSums 
+    calcSums
 
 processReport tz opts (Category cat) = pure (\c -> processCategoryReport opts c cat) <*>
     prepareCalculations
 
-processReport tz opts EachCategory = 
+processReport tz opts EachCategory =
     pure (\c cats -> MultipleReportResults $ map (processCategoryReport opts c) cats) <*>
     prepareCalculations <*>
     onSelected calcCategories
 
 processReport tz opts (IntervalCategory cat) =
-    processIntervalReport tz opts ("Intervals for category " ++ show cat) (extractCat cat) 
+    processIntervalReport tz opts ("Intervals for category " ++ show cat) (extractCat cat)
     where
         extractCat :: Category -> ActivityData -> Maybe String
         extractCat cat = fmap (unpack . activityName) . listToMaybe . filter ( (==Just cat) . activityCategory )
 
 processReport tz opts (IntervalTag tag) =
-    processIntervalReport tz opts ("Intervals for category " ++ show tag) (extractTag tag) 
+    processIntervalReport tz opts ("Intervals for category " ++ show tag) (extractTag tag)
     where
         extractTag :: Activity -> ActivityData -> Maybe String
-        extractTag tag = fmap show . listToMaybe . filter ( (==tag) )
+        extractTag tag = fmap show . listToMaybe . filter (==tag)
 
 processReport tz opts DumpSamples =
     DumpResult <$> onSelected (mapElems toList $ fmap $
@@ -288,41 +283,35 @@ calcCategories = fmap S.toList $ leftFold S.empty $ \s tl ->
           where go' s (Activity (Just cat) _) = S.insert cat s
                 go' s _                       = s
 
-processCategoryReport opts ~(Calculations {..}) cat =
+processCategoryReport opts ~Calculations{..} cat =
         PieChartOfTimePercValues ("Statistics for category " ++ show cat) $
                 let filteredSums = M.filterWithKey (\a _ -> isCategory cat a) sums
-                    uncategorizedTime = totalTimeSel - M.fold (+) 0 filteredSums
+                    uncategorizedTime = totalTimeSel - M.foldl' (+) 0 filteredSums
                     tooSmallSums = M.filter (\t -> realToFrac t / realToFrac totalTimeSel * 100 < roMinPercentage opts) filteredSums
-                    tooSmallTimes = M.fold (+) 0 tooSmallSums
+                    tooSmallTimes = M.foldl' (+) 0 tooSmallSums
                 in
-
-                mapMaybe (\(tag,time) ->
-                      let perc = realToFrac time/realToFrac totalTimeSel
-                          pick = applyActivityFilter (roActivityFilter opts) tag
-                      in if pick && perc*100 >= roMinPercentage opts
-                      then Just ( show tag
-                                , showTimeDiff opts time
-                                , perc)
-                      else Nothing
-                      )
-                      (reverse $ sortBy (comparing snd) $ M.toList filteredSums)
+                [ (show tag, showTimeDiff opts time, perc)
+                | (tag,time) <- sortOn (Down . snd) $ M.toList filteredSums
+                , applyActivityFilter (roActivityFilter opts) tag
+                , let perc = realToFrac time/realToFrac totalTimeSel
+                , perc*100 >= roMinPercentage opts
+                ]
                 ++
-                (
-                if tooSmallTimes > 0
-                then [( printf "(%d entries omitted)" (M.size tooSmallSums)
-                      , showTimeDiff opts tooSmallTimes
-                      , realToFrac tooSmallTimes/realToFrac totalTimeSel
-                      )]
-                else []
-                )
-                ++      
-                (if uncategorizedTime > 0
-                then [( "(unmatched time)"
-                      , showTimeDiff opts uncategorizedTime
-                      , realToFrac uncategorizedTime/realToFrac totalTimeSel
-                      )]
-                else []
-                )
+                [ ( printf "(%d entries omitted)" (M.size tooSmallSums)
+                  , showTimeDiff opts tooSmallTimes
+                  , realToFrac tooSmallTimes/realToFrac totalTimeSel
+                  ) | tooSmallTimes > 0 ]
+                ++
+                [ ( "(unmatched time)"
+                  , showTimeDiff opts uncategorizedTime
+                  , realToFrac uncategorizedTime/realToFrac totalTimeSel
+                  ) | uncategorizedTime > 0]
+                ++
+                [ ( "(total time)"
+                  , showTimeDiff opts totalTimeSel
+                  , realToFrac totalTimeSel/realToFrac totalTimeSel
+                  )
+                ]
 
 tlRateTimediff :: TimeLogEntry a -> NominalDiffTime
 tlRateTimediff tle = fromIntegral (tlRate tle) / 1000
@@ -331,7 +320,7 @@ processIntervalReport :: TimeZone -> ReportOptions -> String -> (ActivityData -
 processIntervalReport tz opts title extr = runOnIntervals  go1 go2
   where
     go1 :: LeftFold (TimeLogEntry (Ctx, ActivityData)) [Interval]
-    go1 = go3 `mapElems` fmap (extr . snd) 
+    go1 = go3 `mapElems` fmap (extr . snd)
     go3 :: LeftFold (TimeLogEntry (Maybe String)) [Interval]
     go3 = runOnGroups sameGroup go4 (onJusts toList)
     sameGroup tl1 tl2 =
@@ -352,10 +341,10 @@ processIntervalReport tz opts title extr = runOnIntervals  go1 go2
         (fromJust <$> lfLast)
     go2 :: LeftFold [Interval] ReportResults
     go2 = ListOfIntervals title <$> concatFold
-        
+
 
 {-
-        ((extr. snd) `filterWith` 
+        ((extr. snd) `filterWith`
             runOnIntervals
                 (runOnGroups ((==) `on` tlData)
 -}
@@ -393,20 +382,20 @@ doRender opts reportdata = case roReportFormat opts of
                 RFCSV -> renderWithDelimiter "," $ renderXSV reportdata
                 RFTSV -> renderWithDelimiter "\t" $ renderXSV reportdata
 
-renderReportText titleMod (ListOfFields title dats) = 
+renderReportText titleMod (ListOfFields title dats) =
     underline (titleMod title) ++
-    (tabulate False $ map (\(f,v) -> [f,v]) dats)
+    tabulate False (map (\(f,v) -> [f,v]) dats)
 
-renderReportText titleMod (ListOfTimePercValues title dats) = 
-    underline (titleMod title) ++ (tabulate True $ listOfValues dats)
+renderReportText titleMod (ListOfTimePercValues title dats) =
+    underline (titleMod title) ++ tabulate True (listOfValues dats)
 
-renderReportText titleMod (PieChartOfTimePercValues title dats) = 
-    underline (titleMod title) ++ (tabulate True $ piechartOfValues dats)
+renderReportText titleMod (PieChartOfTimePercValues title dats) =
+    underline (titleMod title) ++ tabulate True (piechartOfValues dats)
 
-renderReportText titleMod (ListOfIntervals title dats) = 
-    underline (titleMod title) ++ (tabulate True $ listOfIntervals dats)
+renderReportText titleMod (ListOfIntervals title dats) =
+    underline (titleMod title) ++ tabulate True (listOfIntervals dats)
 
-renderReportText titleMod (RepeatedReportResults cat reps) = 
+renderReportText titleMod (RepeatedReportResults cat reps) =
     intercalate "\n" $ map (\(v,rr) -> renderReportText (titleMod . mod v) rr) reps
   where mod v s = s ++ " (" ++ cat ++ " " ++ v ++ ")"
 
@@ -424,7 +413,7 @@ listOfIntervals dats =
 
 -- The reporting of "General Information" is not supported for the
 -- comma-separated output format.
-renderXSV (ListOfFields title dats) = 
+renderXSV (ListOfFields title dats) =
     error ("\"" ++ title ++ "\"" ++ " not supported for this output format")
 
 renderXSV (ListOfTimePercValues _ dats) = listOfValues dats
@@ -462,7 +451,7 @@ showTimeDiffHuman t = go False $ zip [days,hours,mins,secs] ["d","h","m","s"]
         days  =  s `div` (24*60*60)
         hours = (s `div` (60*60)) `mod` 24
         mins  = (s `div` 60) `mod` 60
-        secs  =  s `mod` 60 
+        secs  =  s `mod` 60
         go False []         = "0s"
         go True  []         = ""
 --      go True  vs         | all (==0) (map fst vs) = concat (replicate (length vs) "   ")
@@ -475,13 +464,13 @@ showTimeDiffMachine t = printf "%d:%02d:%02d" hours mins secs
   where s = round t :: Integer
         hours = s `div` (60*60)
         mins  = (s `div` 60) `mod` 60
-        secs  =  s `mod` 60 
+        secs  =  s `mod` 60
 
 showAsLocalTime :: TimeZone -> UTCTime -> String
 showAsLocalTime tz = formatTime defaultTimeLocale "%x %X" . utcToZonedTime tz
 
 underline :: String -> String
-underline str = unlines 
+underline str = unlines
     [ str
     , map (const '=') str
     ]
